# outer __init__.py

__version__ = '0.7.2'

__api_mpy_path__ = 'data/api/mpython.json'
__api_node_path__ = 'data/api/node.json'
__serial_config_path__ = 'data/serial/config.json'

import pyesp.pyesp
import pyesp.run
